from ipykernel.comm import Comm

class CommStringIO(object):
    """ CommStringIO """
    def __init__(self, name=None):
        self.comm = Comm(target_name=name)
        self.buffer = []
        self.maxbuf = 10000

    def close(self):
        """ Closes the CommStringIO. """
        self.flush()
        self.comm.close()

    def isatty(self):
        """Returns false. Just for compatibility to IO."""
        return False

    def __iter__(self):
        """
        Raises exception. Just for compatibility to IO.
        See https://github.com/pandas-dev/pandas/blob/3c0cf22/pandas/core/dtypes/inference.py#L186
        """
        raise Exception('__iter__ called unexpectedly in CommStringIO')

    def write(self, newstr):
        """Appends the given string to the underlying buffer string. Buffer is flushed
           every maxbuf entries."""
        self.buffer.append(newstr)
        if len(self.buffer) >= self.maxbuf:
            self.flush()

    def flush(self):
        """Flushes any buffer entries"""
        if len(self.buffer) > 0:
            self.comm.send(data=''.join(self.buffer))
            self.buffer = []
        return
